Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@sentry/core");
const node_1 = require("@sentry/node");
const types_1 = require("@sentry/types");
const utils_1 = require("@sentry/utils");
const electron_1 = require("electron");
const url = require("url");
const backend_1 = require("../backend");
/** Using net module of electron */
class NetTransport extends node_1.Transports.BaseTransport {
    /** Create a new instance and set this.agent */
    constructor(options) {
        super(options);
        this.options = options;
        /** A simple buffer holding all requests. */
        this._buffer = new utils_1.PromiseBuffer(30);
        /** Locks transport after receiving 429 response */
        this._netDisabledUntil = new Date(Date.now());
    }
    /**
     * @inheritDoc
     */
    sendEvent(event) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line
            if (new Date(Date.now()) < this._netDisabledUntil) {
                return Promise.reject(new utils_1.SentryError(`Transport locked till ${this._netDisabledUntil.toString()} due to too many requests.`));
            }
            if (!this._buffer.isReady()) {
                return Promise.reject(new utils_1.SentryError('Not adding Promise due to buffer limit reached.'));
            }
            yield backend_1.isAppReady();
            return this._buffer.add(new Promise((resolve, reject) => {
                const sentryReq = core_1.eventToSentryRequest(event, this._api);
                const options = this._getRequestOptions(new url.URL(sentryReq.url));
                const req = electron_1.net.request(options);
                req.on('error', reject);
                req.on('response', (res) => {
                    if (res.statusCode && res.statusCode >= 200 && res.statusCode < 300) {
                        resolve({
                            status: types_1.Status.fromHttpCode(res.statusCode),
                        });
                    }
                    else {
                        if (status === types_1.Status.RateLimit) {
                            const now = Date.now();
                            let header = res.headers ? res.headers['Retry-After'] : '';
                            header = Array.isArray(header) ? header[0] : header;
                            this._netDisabledUntil = new Date(now + utils_1.parseRetryAfterHeader(now, header));
                            utils_1.logger.warn(`Too many requests, backing off till: ${this._netDisabledUntil.toString()}`);
                        }
                        // tslint:disable:no-unsafe-any
                        if (res.headers && res.headers['x-sentry-error']) {
                            let reason = res.headers['x-sentry-error'];
                            if (Array.isArray(reason)) {
                                reason = reason.join(', ');
                            }
                            // tslint:enable:no-unsafe-any
                            reject(new utils_1.SentryError(`HTTP Error (${res.statusCode}): ${reason}`));
                        }
                        else {
                            reject(new utils_1.SentryError(`HTTP Error (${res.statusCode})`));
                        }
                    }
                    // force the socket to drain
                    res.on('data', () => {
                        // Drain
                    });
                    res.on('end', () => {
                        // Drain
                    });
                });
                req.write(JSON.stringify(event));
                req.end();
            }));
        });
    }
}
exports.NetTransport = NetTransport;
//# sourceMappingURL=net.js.map